@@ -14,6 +14,9 @@ apiKey = None
 lab = None
 machines = []
 api = None
+configfile = None
+config = None
+
 def init_api(apiKey):
     global api
     api = HTBAPI(apiKey)
@@ -22,7 +25,7 @@ def init_config():
     """
     Initialize the configuration directory and files. This will touch the file whether it already exists or not.
     """
-    global config, configfile, configdir
+    global config, configfile
 
     config = configparser.ConfigParser()
 
@@ -49,6 +52,18 @@ def load_config():
     if "Auth" not in config.sections():
         config['Auth'] = {'apiKey': '', 'lab':'free'}
         write_config()
+    if "List" not in config.sections():
+        defaultFields = ["id", "name", "os", "rating", "difficulty",
+                         "points", "owned_user", "owned_root", "active"]
+        config['List'] = {'defaultFields': ','.join(defaultFields)}
+        write_config()
+
+    if "Info" not in config.sections():
+        defaultFields = ["id", "name", "os", "rating", "difficulty",
+                         "points", "owned_user", "owned_root", "user_blood", "root_blood", "maker", "maker2", "active"]
+        config['Info'] = {'defaultFields': ','.join(defaultFields)}
+        write_config()
+
     apiKey = config['Auth'].get('apiKey')
     lab = config['Auth'].get('lab')
 
@@ -92,6 +107,16 @@ def get_id(name):
             return machine['id']
     return None
 
+
+def get_box(name):
+    """
+    Finds and returns a box with the given name from the machines list.
+    """
+    for machine in machines:
+        if machine['name'].lower() == name.lower():
+            return machine
+    return None
+
 def format_value(val):
     """
     A helper function to format fields for table output.
@@ -144,8 +169,9 @@ def do_config(args):
     Currently this only includes setting or retrieving the current api key.
     """
     global apiKey, lab, config
-    config['Auth']['apiKey'] = args.apiKey
-    config['Auth']['lab'] = args.lab
+    config['Auth']['apiKey'] = args.apiKey or config['Auth']['apiKey']
+    config['Auth']['lab'] = args.lab or config['Auth']['lab']
+    config['List']['defaultFields'] = ",".join(args.listFields) if args.listFields else config['List']['defaultFields']
     write_config()
     apiKey = args.apiKey
     
@@ -198,7 +224,7 @@ def do_list(args):
     if args.all_fields:
         fields = machines[0].keys() if len(machines) > 0 else []
     else:
-        fields = args.fields or ["id", "name", "os", "rating", "owned_user", "owned_root", "active"]
+        fields = args.fields
 
     machine_ids=None
     if args.assigned:
@@ -219,14 +245,18 @@ def do_info(args):
     Currently only prints out the values of the specified box.
     Eventually, this should include options to better format the output.
     """
-    machine = api.get_machine(get_id(args.box))
+    machine = get_box(args.box)
+    machine.update(api.get_machine(machine['id']))
     if args.all_fields:
         fields = machine.keys()
     else:
         fields = args.fields or machine.keys()
+    
     if args.quiet:
         vals = [ str(machine[f]) for f in fields ]
         print(args.separator.join(vals))
+    elif args.list_format:
+        print_machine_list([machine], fields)
     else:
         print_machine(machine, fields)
 
@@ -311,10 +341,12 @@ def parse_args():
     command_parsers = parser.add_subparsers(title="commands", prog="htb")
 
     config_parser = command_parsers.add_parser("config", help="configure this tool")
-    config_parser.add_argument("--apiKey", type=str, required=True, help="Your HTB api key. You can find this on your HTB settings page. THIS MUST BE SET BEFORE YOU CAN USE THIS TOOL FOR ANYTHING ELSE.")
-    config_parser.add_argument("--lab", type=str, choices=["free", "vip"], required=True, help="Which lab you connect to, either free or vip. If you do not pay for your HTB account then you need to pass free or this wont work.")
+    config_parser.add_argument("--apiKey", type=str, help="Your HTB api key. You can find this on your HTB settings page. THIS MUST BE SET BEFORE YOU CAN USE THIS TOOL FOR ANYTHING ELSE.")
+    config_parser.add_argument("--lab", type=str, choices=["free", "vip"], help="Which lab you connect to, either free or vip. If you do not pay for your HTB account then you need to pass free or this wont work.")
+    config_parser.add_argument("--listFields", type=str, nargs="+", metavar="field", help="The default fields to show in List mode.")
     config_parser.set_defaults(func=do_config, command="config")
 
+    defaultListFields = [x.strip() for x in config['List'].get('defaultFields').split(',')]
     list_parser = command_parsers.add_parser("list", help="list machines")
     list_parser.add_argument("--retired", action="store_true", help="Include retired boxes in the output. [NOTE: Retired boxes are only available to VIP users and cannot be accessed by a free user.]")
     list_parser.add_argument("--assigned", action="store_true", help="Show what machines are assigned to you. [VIP Only]")
@@ -324,16 +356,19 @@ def parse_args():
     list_parser.add_argument("-s", "--separator", type=str, default=" ", help="The separator to use when outputting the fields when -q is set")
     list_parser.add_argument("-rs", "--row-separator", type=str, default="\n", help="The separator to use between rows when outputting the fields when -q is set")
     list_parser.add_argument("-q", "--quiet", action="store_true", help="Output only the field values without any formatting. Useful when parsing the output.")
-    list_parser.add_argument("-f", "--fields", metavar="field", default=["id", "name", "os", "rating","difficulty", "points", "owned_user", "owned_root", "active"], nargs="+", type=str, help="Limit the output to only these fields. All fields shown when this is omitted.")
+    list_parser.add_argument("-f", "--fields", metavar="field", default=defaultListFields, nargs="+", type=str,
+                             help="Limit the output to only these fields. The default fields to show can be set with the config command.")
     list_parser.add_argument("-a", "--all-fields", action="store_true", help="Output every field on the machines.")
     list_parser.set_defaults(func=do_list, command="list")
 
+    defaultInfoFields = [x.strip() for x in config['Info'].get('defaultFields').split(',')]
     info_parser = command_parsers.add_parser("info", help="get info about a machine")
     info_parser.add_argument("box", metavar="BOX",  type=str, help="The name of the box you want info for.")
     info_parser.add_argument("-s", "--separator", type=str, default=" ", help="The separator to use when outputting the fields when -q is set")
     info_parser.add_argument("-q", "--quiet", action="store_true", help="Output only the field values without any formatting. Useful when parsing the output.")
-    info_parser.add_argument("-f", "--fields", metavar="field", nargs="+", type=str, help="Limit the output to only these fields. All fields shown when this is omitted.")
+    info_parser.add_argument("-f", "--fields", metavar="field", nargs="+", type=str, default=defaultInfoFields, help="Limit the output to only these fields. The default fields to show can be set with the config command.")
     info_parser.add_argument("-a", "--all-fields", action="store_true", help="Output every field on the machine.")
+    info_parser.add_argument("-l", "--list-format", action="store_true", help="Display the machine info in the same format as the default list command.")
     info_parser.set_defaults(func=do_info, command="info")
 
     spawn_parser = command_parsers.add_parser("spawn", help="[VIP Only] spawn a machine")
@@ -380,4 +415,4 @@ def main():
 
 
 if __name__ == "__main__":
-    main()
\ No newline at end of file
+    main()
